(******************************************************************************
 * capnp-ocaml
 *
 * Copyright (c) 2013-2014, Paul Pelzl
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)


type ro = MessageSig.ro
type rw = MessageSig.rw

module Make (Storage : MessageStorage.S) :
  (MessageSig.S with type Segment.storage_t = Storage.t
      and type Message.storage_t = Storage.t)

(** [BytesMessage] is the standard message format based on OCaml
    [bytes] buffers. *)
module BytesMessage :
  (MessageSig.S with type Segment.storage_t = Bytes.t
                 and type Message.storage_t = Bytes.t)

(** [Invalid_message] is raised by accessor functions whenever the access cannot
    be completed because the message appears to be ill-formed. *)
exception Invalid_message of string

(** [Out_of_int_range] is raised by convenience accessor functions when a getter
    retrieves a value which cannot be expressed as an OCaml int. *)
exception Out_of_int_range of string

(** [invalid_msg err] is equivalent to [raise (Invalid_message err)]. *)
val invalid_msg : string -> 'a

(** [out_of_int_range err] is equivalent to [raise (Out_of_int_range err)]. *)
val out_of_int_range : string -> 'a

